# Copyright 2015 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from oslo_serialization import jsonutils
from tempest.lib import decorators

from mistral_tempest_tests.tests import base


class OpenStackActionsTestsV2(base.TestCase):

    _service = 'workflowv2'

    # TODO(akuznetsova): add checks for task result after task_output
    # TODO(akuznetsova): refactoring will be finished

    def setUp(self):
        super(OpenStackActionsTestsV2, self).setUp()

        _, self.wb = self.client.create_workbook(
            'openstack/action_collection_wb.yaml')

    @decorators.attr(type='openstack')
    @decorators.idempotent_id('9a999fc2-a089-4375-bc69-e1ed85b17a82')
    def test_nova_actions(self):
        wf_name = self.wb['name'] + '.nova'
        _, execution = self.client.create_execution(wf_name)
        self.client.wait_execution_success(execution)
        executed_tasks = self.client.get_wf_tasks(wf_name)
        for executed_task in executed_tasks:
            self.assertEqual('SUCCESS', executed_task['state'])
            if executed_task['name'] == 'versions_get_current':
                published = jsonutils.loads(executed_task['published'])
                # NOTE(kiennt): By default, microversion is 2.1.
                #               But now Mistral supports OpenStack project
                #               dynamic version, so the version should be
                #               the max possible version.
                self.assertNotEqual('2.1', published['result']['version'])

    @decorators.attr(type='openstack')
    @decorators.idempotent_id('81bdc1c9-cd9a-4c97-b8ce-e44f5211eace')
    def test_keystone_actions(self):
        wf_name = self.wb['name'] + '.keystone'
        _, execution = self.admin_client.create_execution(wf_name)
        self.admin_client.wait_execution_success(execution)
        executed_task = self.admin_client.get_wf_tasks(wf_name)[-1]

        self.assertEqual('SUCCESS', executed_task['state'])

    @decorators.attr(type='openstack')
    @decorators.idempotent_id('fde681b8-3e1b-4172-a4b8-2fcac1f070d9')
    def test_heat_actions(self):
        wf_name = self.wb['name'] + '.heat'
        _, execution = self.client.create_execution(wf_name)
        self.client.wait_execution_success(execution)
        executed_task = self.client.get_wf_tasks(wf_name)[-1]

        self.assertEqual('SUCCESS', executed_task['state'])

    @decorators.attr(type='openstack')
    @decorators.idempotent_id('5981360d-f336-45ca-9d38-799c7a8ade26')
    def test_glance_actions(self):
        wf_name = self.wb['name'] + '.glance'
        _, execution = self.client.create_execution(wf_name)
        self.client.wait_execution_success(execution)
        executed_task = self.client.get_wf_tasks(wf_name)[-1]

        self.assertEqual('SUCCESS', executed_task['state'])

    @decorators.attr(type='openstack')
    @decorators.idempotent_id('a1f71a72-3681-4d32-aad9-117068717b33')
    def test_cinder_actions(self):
        wf_name = self.wb['name'] + '.cinder'
        _, execution = self.client.create_execution(wf_name)
        self.client.wait_execution_success(execution)
        executed_task = self.client.get_wf_tasks(wf_name)[-1]

        self.assertEqual('SUCCESS', executed_task['state'])

    @decorators.attr(type='openstack')
    @decorators.idempotent_id('586dd973-fc65-40e2-9a85-31418b22473a')
    def test_neutron_actions(self):
        wf_name = self.wb['name'] + '.neutron'
        _, execution = self.client.create_execution(wf_name)
        self.client.wait_execution_success(execution)
        executed_task = self.client.get_wf_tasks(wf_name)[-1]

        self.assertEqual('SUCCESS', executed_task['state'])

    @decorators.attr(type='openstack')
    @decorators.idempotent_id('985c1051-cf2e-4fd0-8ceb-a9b8110597a1')
    def test_swift_actions(self):
        wf_name = self.wb['name'] + '.swift'
        _, execution = self.client.create_execution(wf_name)
        self.client.wait_execution_success(execution)
        executed_task = self.client.get_wf_tasks(wf_name)[-1]

        self.assertEqual('SUCCESS', executed_task['state'])



# Copyright 2016 NEC Corporation. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging
from oslo_serialization import jsonutils
from tempest.lib import decorators
from tempest.lib import exceptions

from mistral_tempest_tests.tests import base


LOG = logging.getLogger(__name__)


class ActionExecutionTestsV2(base.TestCase):
    _service = 'workflowv2'

    def setUp(self):
        super(ActionExecutionTestsV2, self).setUp()

        self.client.create_action_execution(
            {
                'name': 'std.echo',
                'input': '{"output": "Hello, Mistral!"}'
            }
        )

    def tearDown(self):
        for action_ex in self.client.action_executions:
            try:
                self.client.delete_obj('action_executions', action_ex)
            except Exception as e:
                LOG.exception(
                    'Exception raised when deleting '
                    'action_executions %s, error message: %s.', action_ex, e
                )

        self.client.action_executions = []

        super(ActionExecutionTestsV2, self).tearDown()

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('a72603bd-5d49-4d92-9747-8da6322e867d')
    def test_run_action_execution(self):
        resp, body = self.client.create_action_execution(
            {
                'name': 'std.echo',
                'input': '{"output": "Hello, Mistral!"}'
            }
        )

        self.assertEqual(201, resp.status)
        output = jsonutils.loads(body['output'])
        self.assertDictEqual(
            {'result': 'Hello, Mistral!'},
            output
        )

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('0623cb62-b20a-45c8-afd9-8da46e1bb3cb')
    def test_list_action_executions(self):
        resp, body = self.client.get_list_obj('action_executions')

        self.assertEqual(200, resp.status)

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('cd36ea00-7e22-4c3d-90c3-fb441b93cf12')
    def test_output_appear_in_response_only_when_needed(self):
        resp, body = self.client.get_list_obj('action_executions')

        self.assertEqual(200, resp.status)
        action_execution = body['action_executions'][0]
        self.assertNotIn("output", action_execution)

        resp, body = self.client.get_list_obj(
            'action_executions?include_output=True'
        )

        self.assertEqual(200, resp.status)
        action_execution = body['action_executions'][0]
        self.assertIn("output", action_execution)

        resp, body = self.client.get_action_execution(action_execution['id'])
        self.assertIn("output", body.decode('utf-8'))

        # Test when passing task execution ID

        resp, body = self.client.create_workflow('wf_v2.yaml')
        wf_name = body['workflows'][0]['name']
        self.assertEqual(201, resp.status)
        resp, body = self.client.create_execution(wf_name)
        self.assertEqual(201, resp.status)
        resp, body = self.client.get_list_obj('tasks')
        self.assertEqual(200, resp.status)
        task_id = body['tasks'][0]['id']

        resp, body = self.client.get_list_obj(
            'action_executions?include_output=true&task_execution_id=%s' %
            task_id
        )

        self.assertEqual(200, resp.status)
        action_execution = body['action_executions'][0]
        self.assertIn("output", action_execution)

        resp, body = self.client.get_list_obj(
            'action_executions?&task_execution_id=%s' %
            task_id
        )

        self.assertEqual(200, resp.status)
        action_execution = body['action_executions'][0]
        self.assertNotIn("output", action_execution)

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('dc76aeda-9243-45cf-bfd2-141d3af8b28b')
    def test_run_action_std_http(self):
        resp, body = self.client.create_action_execution(
            {
                'name': 'std.http',
                'input': '{"url": "http://wiki.openstack.org"}'
            }
        )

        self.assertEqual(201, resp.status)
        output = jsonutils.loads(body['output'])
        self.assertTrue(output['result']['status'] in range(200, 307))

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('befa9b1c-01a4-41bc-b060-88cb1b147dfb')
    def test_run_action_std_http_error(self):
        resp, body = self.client.create_action_execution(
            {
                'name': 'std.http',
                'input': '{"url": "http://www.google.ru/not-found-test"}'
            }
        )

        self.assertEqual(201, resp.status)
        output = jsonutils.loads(body['output'])
        self.assertEqual(404, output['result']['status'])

    @decorators.attr(type='sanity')
    @decorators.related_bug('1667415')
    @decorators.idempotent_id('3c73de7a-4af0-4657-90d6-d7ebd3c7da18')
    def test_run_action_std_http_non_utf8_response(self):
        resp, body = self.client.create_action_execution(
            {
                'name': 'std.http',
                'input':
                    '{"url": "https://httpbin.org/encoding/utf8"}'
            }
        )

        self.assertEqual(201, resp.status)
        output = jsonutils.loads(body['output'])
        self.assertEqual(200, output['result']['status'])

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('d98586bf-fdc4-44f6-9837-700d35b5f889')
    def test_create_action_execution(self):
        resp, body = self.client.create_action_execution(
            {
                'name': 'std.echo',
                'input': '{"output": "Hello, Mistral!"}',
                'params': '{"save_result": true}'
            }
        )

        self.assertEqual(201, resp.status)
        self.assertEqual('RUNNING', body['state'])

        # We must reread action execution in order to get actual
        # state and output.
        body = self.client.wait_execution_success(
            body,
            url='action_executions'
        )
        output = jsonutils.loads(body['output'])

        self.assertEqual('SUCCESS', body['state'])
        self.assertDictEqual(
            {'result': 'Hello, Mistral!'},
            output
        )

    @decorators.attr(type='negative')
    @decorators.idempotent_id('99f22c17-6fb4-4480-96d3-4a82672916b7')
    def test_delete_nonexistent_action_execution(self):
        self.assertRaises(
            exceptions.NotFound,
            self.client.delete_obj,
            'action_executions',
            'nonexist'
        )

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('2dbd74ba-4950-4c52-8bd3-070d634dcd05')
    def test_create_action_execution_sync(self):
        resp, body = self.client.create_action_execution(
            {
                'name': 'std.echo',
                'input': '{"output": "Hello Tempest"}'
            }
        )

        self.assertEqual(201, resp.status)
        output = jsonutils.loads(body['output'])
        self.assertEqual("Hello Tempest", output['result'])

    @decorators.idempotent_id('9438e195-031c-4502-b216-6d72941ec281')
    @decorators.attr(type='sanity')
    def test_action_execution_of_workflow_within_namespace(self):

        resp, body = self.client.create_workflow('wf_v2.yaml', namespace='abc')
        wf_name = "wf"
        wf_namespace = body['workflows'][0]['namespace']
        self.assertEqual(201, resp.status)
        resp, execution = self.client.create_execution(
            wf_name,
            wf_namespace=wf_namespace
        )
        self.client.wait_execution_success(execution)
        self.assertEqual(201, resp.status)
        resp, body = self.client.get_list_obj('tasks')
        self.assertEqual(200, resp.status)
        task_id = body['tasks'][0]['id']

        resp, body = self.client.get_list_obj(
            'action_executions?include_output=true&task_execution_id=%s' %
            task_id)

        self.assertEqual(200, resp.status)
        action_execution = body['action_executions'][0]
        self.assertEqual(wf_namespace, action_execution['workflow_namespace'])



# Copyright 2016 NEC Corporation. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_concurrency.fixture import lockutils
from oslo_serialization import jsonutils
from tempest.lib import decorators
from tempest.lib import exceptions

from mistral_tempest_tests.tests import base
from mistral_tempest_tests.tests import utils


class WorkflowTestsV2(base.TestCase):

    _service = 'workflowv2'

    def tearDown(self):
        for wf in self.client.workflows:
            try:
                self.client.delete_obj('workflows', wf)
            except exceptions.NotFound:
                pass

        self.client.workflows = []

        super(WorkflowTestsV2, self).tearDown()

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('e9cd6817-e8d1-4604-ba76-b0e17219f4c5')
    def test_get_list_workflows(self):
        resp, body = self.client.get_list_obj('workflows')
        self.assertEqual(200, resp.status)

        names = [wf['name'] for wf in body['workflows']]

        self.assertPartIn('create_instance', names)

        self.assertNotIn('next', body)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('be8a4a44-eeb3-48e3-b11d-b83ba14dbf2c')
    def test_get_list_workflows_by_admin(self):
        self.useFixture(lockutils.LockFixture('mistral-workflow'))

        _, body = self.client.create_workflow('wf_v2.yaml')
        name = body['workflows'][0]['name']

        resp, raw_body = self.admin_client.get('workflows?all_projects=true')
        body = jsonutils.loads(raw_body)

        self.assertEqual(200, resp.status)

        names = [wf['name'] for wf in body['workflows']]

        self.assertIn(name, names)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('c9e2ebbc-02aa-4c33-b244-e471c8266aa7')
    def test_get_list_workflows_with_project_by_admin(self):
        self.useFixture(lockutils.LockFixture('mistral-workflow'))

        _, body = self.client.create_workflow('wf_v2.yaml')

        name = body['workflows'][0]['name']

        resp, raw_body = self.admin_client.get(
            'workflows?project_id=%s' %
            self.client.auth_provider.credentials.tenant_id
        )
        body = jsonutils.loads(raw_body)

        self.assertEqual(200, resp.status)

        names = [wf['name'] for wf in body['workflows']]

        self.assertIn(name, names)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('b8dc1b02-8509-45e2-9df7-4630cdcfa1ab')
    def test_get_list_other_project_private_workflows(self):
        self.useFixture(lockutils.LockFixture('mistral-workflow'))

        _, body = self.client.create_workflow('wf_v2.yaml')

        name = body['workflows'][0]['name']

        resp, raw_body = self.alt_client.get(
            'workflows?project_id=%s' %
            self.client.auth_provider.credentials.tenant_id
        )
        body = jsonutils.loads(raw_body)

        self.assertEqual(200, resp.status)

        names = [wf['name'] for wf in body['workflows']]

        self.assertNotIn(name, names)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('2063143b-ced8-4037-9383-e2504be581e6')
    def test_get_list_workflows_with_fields(self):
        resp, body = self.client.get_list_obj('workflows?fields=name')

        self.assertEqual(200, resp.status)

        for wf in body['workflows']:
            self.assertListEqual(sorted(['id', 'name']), sorted(list(wf)))

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('81f28735-e74e-4dc1-8b94-b548f8a80556')
    def test_get_list_workflows_with_pagination(self):
        resp, body = self.client.get_list_obj(
            'workflows?limit=1&sort_keys=name&sort_dirs=desc'
        )

        self.assertEqual(200, resp.status)
        self.assertEqual(1, len(body['workflows']))
        self.assertIn('next', body)

        name_1 = body['workflows'][0].get('name')
        next_ = body.get('next')

        param_dict = utils.get_dict_from_string(
            next_.split('?')[1],
            delimiter='&'
        )

        # NOTE: 'id' gets included into sort keys automatically with 'desc'
        # sorting to avoid pagination looping.
        expected_sub_dict = {
            'limit': 1,
            'sort_keys': 'name,id',
            'sort_dirs': 'desc,asc'
        }

        self.assertEqual(param_dict, param_dict | expected_sub_dict)

        # Query again using 'next' hint
        url_param = next_.split('/')[-1]
        resp, body = self.client.get_list_obj(url_param)

        self.assertEqual(200, resp.status)
        self.assertEqual(1, len(body['workflows']))

        name_2 = body['workflows'][0].get('name')

        self.assertGreater(name_1, name_2)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('cdb5586f-a72f-4371-88d1-1472675915c3')
    def test_get_list_workflows_nonexist_sort_dirs(self):
        context = self.assertRaises(
            exceptions.BadRequest,
            self.client.get_list_obj,
            'workflows?limit=1&sort_keys=id&sort_dirs=nonexist'
        )

        self.assertIn(
            'Unknown sort direction',
            context.resp_body.get('faultstring')
        )

    @decorators.attr(type='negative')
    @decorators.idempotent_id('ac41a0ea-2be6-4307-9003-6b8dd52b0bf9')
    def test_get_list_workflows_invalid_limit(self):
        context = self.assertRaises(
            exceptions.BadRequest,
            self.client.get_list_obj,
            'workflows?limit=-1&sort_keys=id&sort_dirs=asc'
        )

        self.assertIn(
            'Limit must be positive',
            context.resp_body.get('faultstring')
        )

    @decorators.attr(type='negative')
    @decorators.idempotent_id('55759713-a8d7-44c2-aff1-2383f51136bd')
    def test_get_list_workflows_duplicate_sort_keys(self):
        context = self.assertRaises(
            exceptions.BadRequest,
            self.client.get_list_obj,
            'workflows?limit=1&sort_keys=id,id&sort_dirs=asc,asc'
        )

        self.assertIn(
            'Length of sort_keys must be equal or greater than sort_dirs',
            context.resp_body.get('faultstring')
        )

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('e26b30b9-6699-4020-93a0-e25c2daca59a')
    def test_create_and_delete_workflow(self):
        self.useFixture(lockutils.LockFixture('mistral-workflow'))
        resp, body = self.client.create_workflow('wf_v2.yaml')
        name = body['workflows'][0]['name']

        self.assertEqual(201, resp.status)

        resp, body = self.client.get_list_obj('workflows')
        self.assertEqual(200, resp.status)

        names = [wf['name'] for wf in body['workflows']]
        self.assertIn(name, names)

        self.client.delete_obj('workflows', name)

        _, body = self.client.get_list_obj('workflows')

        names = [wf['name'] for wf in body['workflows']]
        self.assertNotIn(name, names)

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('f5a4a771-79b2-4f28-bfac-940aa83990a4')
    def test_get_workflow(self):
        self.useFixture(lockutils.LockFixture('mistral-workflow'))
        _, body = self.client.create_workflow('wf_v2.yaml')
        name = body['workflows'][0]['name']

        resp, body = self.client.get_object('workflows', name)

        self.assertEqual(200, resp.status)
        self.assertEqual(name, body['name'])

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('f516aad0-9a50-4ace-a217-fa1931fd9335')
    def test_update_workflow(self):
        self.useFixture(lockutils.LockFixture('mistral-workflow'))
        _, body = self.client.create_workflow('single_wf.yaml')
        name = body['workflows'][0]['name']

        resp, body = self.client.update_request('workflows', 'single_wf.yaml')

        self.assertEqual(200, resp.status)
        self.assertEqual(name, body['workflows'][0]['name'])

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('42f5d135-a2b8-4a31-8135-c5ce8c5f1ed5')
    def test_workflow_within_namespace(self):
        self.useFixture(lockutils.LockFixture('mistral-workflow'))

        namespace = 'abc'
        resp, body = self.client.create_workflow(
            'single_wf.yaml',
            namespace=namespace
        )
        name = body['workflows'][0]['name']
        wf_id = body['workflows'][0]['id']

        self.assertEqual(201, resp.status)
        self.assertEqual(name, body['workflows'][0]['name'])

        resp, body = self.client.get_workflow(
            wf_id
        )

        self.assertEqual(namespace, body['namespace'])

        resp, body = self.client.update_workflow('single_wf.yaml', namespace)

        self.assertEqual(200, resp.status)
        self.assertEqual(name, body['workflows'][0]['name'])
        self.assertEqual(namespace, body['workflows'][0]['namespace'])

        namespace = 'abc2'
        resp, body = self.client.create_workflow(
            'single_wf.yaml',
            namespace=namespace
        )
        name = body['workflows'][0]['name']
        wf_id = body['workflows'][0]['id']

        self.assertEqual(201, resp.status)
        self.assertEqual(name, body['workflows'][0]['name'])

        resp, body = self.client.get_workflow(wf_id)

        self.assertEqual(namespace, body['namespace'])

        self.assertRaises(exceptions.NotFound, self.client.get_workflow, name)

        self.client.create_workflow(
            'single_wf.yaml'
        )

        resp, body = self.client.get_workflow(wf_id)
        self.assertEqual(200, resp.status)

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('02bc1fc3-c31a-4e37-bb3d-eda46818505c')
    def test_get_workflow_definition(self):
        self.useFixture(lockutils.LockFixture('mistral-workflow'))
        _, body = self.client.create_workflow('wf_v2.yaml')
        name = body['workflows'][0]['name']

        resp, body = self.client.get_definition('workflows', name)

        self.assertEqual(200, resp.status)
        self.assertIsNotNone(body)

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('04fbd003-0e52-4034-858e-6634d4f84b29')
    def test_get_workflow_uploaded_in_wb(self):
        self.useFixture(lockutils.LockFixture('mistral-workflow'))
        _, body = self.client.create_workbook('wb_v2.yaml')
        wb_name = body['name']

        _, body = self.client.get_list_obj('workflows')
        wf_names = [wf['name'] for wf in body['workflows']
                    if wf['name'].startswith(wb_name)]

        self.assertNotEmpty(wf_names)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('5e5f0403-fb2c-41ae-bf6f-25c181515358')
    def test_get_nonexistent_workflow_definition(self):
        self.assertRaises(exceptions.NotFound,
                          self.client.get_definition,
                          'workflows', 'nonexist')

    @decorators.attr(type='negative')
    @decorators.idempotent_id('23c72d01-c3bb-43d6-ba15-9b49c15f800c')
    def test_get_nonexistent_workflow(self):
        self.assertRaises(exceptions.NotFound, self.client.get_object,
                          'workflows', 'nonexist')

        exception = self.assertRaises(
            exceptions.NotFound,
            self.client.get_workflow,
            'nonexist_wf',
            'nonexist_namespace'
        )
        self.assertIn('nonexist_wf', str(exception))
        self.assertIn('nonexist_namespace', str(exception))

    @decorators.attr(type='negative')
    @decorators.idempotent_id('6b917213-7f11-423a-8fe0-55795dcf0fb2')
    def test_double_create_workflows(self):
        self.useFixture(lockutils.LockFixture('mistral-workflow'))
        _, body = self.client.create_workflow('wf_v2.yaml')
        self.assertRaises(exceptions.Conflict,
                          self.client.create_workflow,
                          'wf_v2.yaml')

    @decorators.attr(type='negative')
    @decorators.idempotent_id('ffcd63d2-1104-4320-a67b-fadc4e2a0631')
    def test_create_wf_with_invalid_def(self):
        self.assertRaises(exceptions.BadRequest,
                          self.client.create_workflow,
                          'wb_v1.yaml')

    @decorators.attr(type='negative')
    @decorators.idempotent_id('eed46931-5485-436c-810f-1f63362223b9')
    def test_update_wf_with_invalid_def(self):
        self.assertRaises(exceptions.BadRequest,
                          self.client.update_request,
                          'workflows', 'wb_v1.yaml')

    @decorators.attr(type='negative')
    @decorators.idempotent_id('9b7f5b5a-cacd-4f98-a35a-decf065b8234')
    def test_delete_wf_with_trigger_associate(self):
        tr_name = 'trigger'
        resp, body = self.client.create_workflow('wf_v2.yaml')
        name = body['workflows'][0]['name']
        self.client.create_cron_trigger(tr_name, name, None, '5 * * * *')

        try:
            self.assertRaises(
                exceptions.BadRequest,
                self.client.delete_obj,
                'workflows',
                name
            )
        finally:
            self.client.delete_obj('cron_triggers', tr_name)
            self.client.triggers.remove(tr_name)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('46325022-cbd2-48f3-95f3-e587aab3b655')
    def test_delete_wf_with_event_trigger_associate(self):
        _, body = self.client.create_workflow('wf_v2.yaml')
        wf_id = body['workflows'][0]['id']
        resp, body = self.client.create_event_trigger(
            wf_id, 'openstack', 'notification', 'fake.event')
        self.assertEqual(201, resp.status)

        try:
            self.assertRaises(
                exceptions.BadRequest,
                self.client.delete_obj,
                'workflows',
                wf_id
            )
        finally:
            self.client.delete_obj('event_triggers', body['id'])
            self.client.event_triggers.remove(body['id'])

    @decorators.attr(type='negative')
    @decorators.idempotent_id('1cb929e6-d375-4dcb-ab7c-73aa205af896')
    def test_delete_wf_with_trigger_associate_in_other_tenant(self):
        self.useFixture(lockutils.LockFixture('mistral-workflow'))
        tr_name = 'trigger'
        _, body = self.client.create_workflow('wf_v2.yaml', scope='public')
        name = body['workflows'][0]['name']
        self.alt_client.create_cron_trigger(tr_name, name, None, '5 * * * *')

        try:
            exception = self.assertRaises(
                exceptions.BadRequest,
                self.client.delete_obj,
                'workflows',
                name
            )

            self.assertIn(
                "Can't delete workflow that has cron triggers associated",
                exception.resp_body['faultstring']
            )
        finally:
            self.alt_client.delete_obj('cron_triggers', tr_name)
            self.alt_client.triggers.remove(tr_name)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('f575713b-27fd-4ec8-b84f-468a7adf5ed2')
    def test_delete_nonexistent_wf(self):
        self.assertRaises(exceptions.NotFound,
                          self.client.delete_obj,
                          'workflows', 'nonexist')



# Copyright 2016 NEC Corporation. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_concurrency.fixture import lockutils
from tempest.lib import decorators

from mistral_tempest_tests.tests import base


class TasksTestsV2(base.TestCase):

    _service = 'workflowv2'

    def setUp(self):
        super(TasksTestsV2, self).setUp()
        self.useFixture(lockutils.LockFixture('mistral-workflow'))
        _, body = self.client.create_workflow('wf_v2.yaml')
        self.direct_wf_name = body['workflows'][0]['name']
        _, execution = self.client.create_execution(self.direct_wf_name)
        self.execution_id = execution['id']

    def tearDown(self):
        for wf in self.client.workflows:
            self.client.delete_obj('workflows', wf)
        self.client.workflows = []

        for wf in self.client.executions:
            self.client.delete_obj('executions', wf, force=True)
        self.client.executions = []

        super(TasksTestsV2, self).tearDown()

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('81159dce-3802-44ee-a8d4-5ddca106fd91')
    def test_get_tasks_list(self):
        resp, body = self.client.get_list_obj('tasks')

        self.assertEqual(200, resp.status)
        self.assertNotEmpty(body['tasks'])

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('f62664de-bd2b-4153-8d0f-5a76d78abbad')
    def test_get_task(self):
        resp, body = self.client.get_list_obj('tasks')

        self.assertEqual(200, resp.status)
        task_id = body['tasks'][-1]['id']
        resp, body = self.client.get_object('tasks', task_id)
        self.assertEqual(200, resp.status)
        self.assertEqual(self.direct_wf_name, body['workflow_name'])

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('3230d694-40fd-4094-ad12-024f40a21b94')
    def test_get_tasks_of_execution(self):
        resp, body = self.client.get_list_obj(
            'tasks?workflow_execution_id=%s' % self.execution_id)

        self.assertEqual(200, resp.status)
        self.assertEqual(
            self.direct_wf_name, body['tasks'][-1]['workflow_name']
        )


class TaskTypesTestsV2(base.TestCase):

    _service = 'workflowv2'

    def setUp(self):
        super(TaskTypesTestsV2, self).setUp()

        self.useFixture(lockutils.LockFixture('mistral-workflow'))

        _, wb_body = self.client.create_workbook('wb_with_nested_wf.yaml')
        self.nested_wf_name = 'wb_with_nested_wf.wrapping_wf'
        _, execution = self.client.create_execution(self.nested_wf_name)
        self.client.wait_execution_success(execution)

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('1ac726eb-b945-4b82-8755-a2fb2dc009bc')
    def test_task_type(self):
        resp, body = self.client.get_list_obj('tasks')

        self.assertEqual(200, resp.status)

        bt = body['tasks']
        ll = [[v for k, v in d.items() if 'type' in k] for d in bt]
        types_list = [item for sublist in ll for item in sublist]

        self.assertIn(
            'WORKFLOW', types_list
        )
        self.assertIn(
            'ACTION', types_list
        )

        # there are 2 tasks in the workflow one of each type
        self.assertEqual(
            2, len(types_list)
        )



# Copyright 2016 - Nokia, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
import base64

from keystoneclient import service_catalog as ks_service_catalog
from oslo_serialization import jsonutils
from oslo_utils import uuidutils
from tempest.lib import decorators
from urllib.parse import urlparse

from mistral_tempest_tests.tests import base


class MultiVimActionsTests(base.TestCase):
    _service = 'workflowv2'

    @decorators.attr(type='openstack')
    @decorators.idempotent_id('dadc9960-9c03-41a9-9a9d-7e97d527e6dd')
    @decorators.skip_because(bug="1964344")
    def test_multi_vim_support_target_headers(self):
        client_1 = self.alt_client
        client_2 = self.client

        # Create stack with client2.
        result = _execute_action(client_2, _get_create_stack_request())
        stack_id = str(
            jsonutils.loads(result['output'])['result']['stack']['id']
        )

        # List stacks with client1, and assert that there is no stack.
        result = _execute_action(client_1, _get_list_stack_request())
        self.assertEmpty(jsonutils.loads(result['output'])['result'])

        # List stacks with client1, but with the target headers of client2,
        # and assert the created stack is there.
        result = _execute_action(
            client_1,
            _get_list_stack_request(),
            extra_headers=_extract_target_headers_from_client(client_2)
        )
        self.assertEqual(
            stack_id,
            str(jsonutils.loads(result['output'])['result'][0]['id'])
        )

    @decorators.attr(type='openstack')
    @decorators.idempotent_id('bc0e9b99-62b0-4d96-95c9-016a3f69b02a')
    @decorators.skip_because(bug="1736685")
    def test_multi_vim_support_target_headers_and_service_catalog(self):
        client_1 = self.alt_client
        client_2 = self.client

        # List stacks with client1, but with the target headers of client2,
        # and additionally with an invalid X-Target-Service-Catalog.
        extra_headers = _extract_target_headers_from_client(client_2)

        # Use ServiceCatalog to eliminate differences between keystone v2 and
        # v3.
        token_data = client_2.auth_provider.auth_data[1]
        service_catalog = ks_service_catalog.ServiceCatalog.factory(
            token_data
        ).get_data()

        for service in service_catalog:
            if service['name'] == 'heat':
                for ep in service['endpoints']:
                    if 'publicURL' in ep:
                        ep['publicURL'] = "invalid"
                    elif ep['interface'] == 'public':
                        ep['url'] = "invalid"
                        break
                break

        if 'catalog' in token_data:
            token_data['catalog'] = service_catalog
        else:
            token_data['serviceCatalog'] = service_catalog

        invalid_service_catalog = {
            "X-Target-Service-Catalog": base64.b64encode(
                jsonutils.dumps(token_data)
            )
        }
        extra_headers.update(invalid_service_catalog)
        result = _execute_action(
            client_1,
            _get_list_stack_request(),
            extra_headers=extra_headers
        )

        # Assert that the invalid catalog was used.
        self.assertIn("Invalid URL", result['output'])


def _extract_target_headers_from_client(client):
    u = urlparse(client.auth_provider.auth_url)
    v3_auth_url = '{}://{}/identity/v3/'.format(u.scheme, u.netloc)
    return {
        'X-Target-Auth-Token': client.token,
        'X-Target-Auth-Uri': v3_auth_url,
        'X-Target-Project-Id': client.tenant_id,
        'X-Target-User-Id': client.user_id,
    }


def _execute_action(client, request, extra_headers=None):
    if extra_headers is None:
        extra_headers = {}
    _, result = client.create_action_execution(
        request,
        extra_headers=extra_headers
    )

    return result


def _get_create_stack_request():
    stack_name = 'multi_vim_test_stack_{}'.format(
        uuidutils.generate_uuid()[:8])

    return {
        'name': 'heat.stacks_create',
        'input': {
            'stack_name': stack_name,
            "template": {"heat_template_version": "2013-05-23"}
        }
    }


def _get_list_stack_request():
    return {
        'name': 'heat.stacks_list',
    }



# Copyright 2016 Catalyst IT Limited
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg

ServiceAvailableGroup = [cfg.BoolOpt('mistral',
                                     default=True,
                                     help="Whether or not Mistral is expected"
                                          " to be available")]

service_available_group = cfg.OptGroup(name="service_available",
                                       title="Available OpenStack Services")

mistral_api_group = cfg.OptGroup(name="mistral_api",
                                 title="Mistral Api Service Options")

MistralApiGroup = [cfg.BoolOpt("service_api_supported",
                               default=False,
                               help="Whether or not services api "
                                    "is available")]



#  Copyright 2019 - Nokia Corporation
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.


from tempest import config
from tempest.lib import decorators

from mistral_tempest_tests.tests import base

CONF = config.CONF


class ServicesTestsV2(base.TestCase):

    _service = 'workflowv2'

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('f4359ad2-9109-4305-a00a-77679878f7f9')
    @decorators.skip_because(bug="2099954")
    def test_get_services_list(self):
        resp, body = self.client.get_list_obj('services')

        self.assertEqual(200, resp.status)
        self.assertNotEmpty(body['services'])



# Copyright 2016 NEC Corporation. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_concurrency.fixture import lockutils
from tempest.lib import decorators
from tempest.lib import exceptions

from mistral_tempest_tests.tests import base


class CronTriggerTestsV2(base.TestCase):

    _service = 'workflowv2'

    def setUp(self):
        super(CronTriggerTestsV2, self).setUp()
        self.useFixture(lockutils.LockFixture('mistral-workflow'))
        _, body = self.client.create_workflow('wf_v2.yaml')
        self.wf_name = body['workflows'][0]['name']

    def tearDown(self):

        for tr in self.client.triggers:
            self.client.delete_obj('cron_triggers', tr)
        self.client.triggers = []

        for wf in self.client.workflows:
            self.client.delete_obj('workflows', wf)
        self.client.workflows = []

        super(CronTriggerTestsV2, self).tearDown()

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('c53b44dd-59b3-4a4b-b22a-21abb4cecea0')
    def test_get_list_cron_triggers(self):
        resp, body = self.client.get_list_obj('cron_triggers')

        self.assertEqual(200, resp.status)
        self.assertEmpty(body['cron_triggers'])

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('fbc641fa-8704-45b3-b259-136eb956394c')
    def test_create_and_delete_cron_triggers(self):
        tr_name = 'trigger'

        resp, body = self.client.create_cron_trigger(
            tr_name, self.wf_name, None, '5 * * * *')
        self.assertEqual(201, resp.status)
        self.assertEqual(tr_name, body['name'])

        resp, body = self.client.get_list_obj('cron_triggers')
        self.assertEqual(200, resp.status)

        trs_names = [tr['name'] for tr in body['cron_triggers']]
        self.assertIn(tr_name, trs_names)

        self.client.delete_obj('cron_triggers', tr_name)
        self.client.triggers.remove(tr_name)

        _, body = self.client.get_list_obj('cron_triggers')

        trs_names = [tr['name'] for tr in body['cron_triggers']]
        self.assertNotIn(tr_name, trs_names)

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('b8b9102e-b323-492f-af41-4f5368971e36')
    def test_create_and_delete_oneshot_cron_triggers(self):
        tr_name = 'trigger'

        resp, body = self.client.create_cron_trigger(
            tr_name, self.wf_name, None, None, "4242-12-25 13:37")
        self.assertEqual(201, resp.status)
        self.assertEqual(tr_name, body['name'])
        self.assertEqual("4242-12-25 13:37:00", body['next_execution_time'])

        resp, body = self.client.get_list_obj('cron_triggers')
        self.assertEqual(200, resp.status)

        trs_names = [tr['name'] for tr in body['cron_triggers']]
        self.assertIn(tr_name, trs_names)

        self.client.delete_obj('cron_triggers', tr_name)
        self.client.triggers.remove(tr_name)

        _, body = self.client.get_list_obj('cron_triggers')

        trs_names = [tr['name'] for tr in body['cron_triggers']]
        self.assertNotIn(tr_name, trs_names)

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('5224359b-3c31-4fe7-a4eb-dc9da843137e')
    def test_create_two_cron_triggers_for_one_wf(self):
        tr_name_1 = 'trigger1'
        tr_name_2 = 'trigger2'

        resp, body = self.client.create_cron_trigger(
            tr_name_1, self.wf_name, None, '5 * * * *')
        self.assertEqual(201, resp.status)
        self.assertEqual(tr_name_1, body['name'])

        resp, body = self.client.create_cron_trigger(
            tr_name_2, self.wf_name, None, '15 * * * *')
        self.assertEqual(201, resp.status)
        self.assertEqual(tr_name_2, body['name'])

        resp, body = self.client.get_list_obj('cron_triggers')
        self.assertEqual(200, resp.status)

        trs_names = [tr['name'] for tr in body['cron_triggers']]
        self.assertIn(tr_name_1, trs_names)
        self.assertIn(tr_name_2, trs_names)

    @decorators.attr(type='sanity')
    @decorators.idempotent_id('967da6e3-f9a2-430a-9390-0d73f2143aba')
    def test_get_cron_trigger(self):
        tr_name = 'trigger'
        self.client.create_cron_trigger(
            tr_name, self.wf_name, None, '5 * * * *')

        resp, body = self.client.get_object('cron_triggers', tr_name)

        self.assertEqual(200, resp.status)
        self.assertEqual(tr_name, body['name'])

    @decorators.attr(type='negative')
    @decorators.idempotent_id('d0e4d894-9a50-4919-a008-a9f255b6b6a3')
    def test_create_cron_trigger_nonexistent_wf(self):
        self.assertRaises(exceptions.NotFound,
                          self.client.create_cron_trigger,
                          'trigger', 'nonexist', None, '5 * * * *')

    @decorators.attr(type='negative')
    @decorators.idempotent_id('83f0d420-fd3c-4e75-87b1-854cefb28bda')
    def test_create_cron_trigger_invalid_count(self):
        self.assertRaises(exceptions.BadRequest,
                          self.client.create_cron_trigger,
                          'trigger', 'nonexist', None, '5 * * * *', None, "q")

    @decorators.attr(type='negative')
    @decorators.idempotent_id('4190e0af-3c64-4f57-a0b8-d9d3d41fd323')
    def test_create_cron_trigger_negative_count(self):
        self.assertRaises(exceptions.BadRequest,
                          self.client.create_cron_trigger,
                          'trigger', 'nonexist', None, '5 * * * *', None, -1)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('210c37e8-990e-4260-b3b3-93f254e6a4d7')
    def test_create_cron_trigger_invalid_first_date(self):
        self.assertRaises(exceptions.BadRequest,
                          self.client.create_cron_trigger,
                          'trigger', 'nonexist', None, '5 * * * *', "q")

    @decorators.attr(type='negative')
    @decorators.idempotent_id('17990a39-8f66-4748-8ba5-ca87befbb198')
    def test_create_cron_trigger_count_only(self):
        self.assertRaises(exceptions.BadRequest,
                          self.client.create_cron_trigger,
                          'trigger', 'nonexist', None, None, None, "42")

    @decorators.attr(type='negative')
    @decorators.idempotent_id('029e0a1e-2252-4a37-b9bd-cfbe407c6ade')
    def test_create_cron_trigger_date_and_count_without_pattern(self):
        self.assertRaises(exceptions.BadRequest,
                          self.client.create_cron_trigger,
                          'trigger', 'nonexist', None, None,
                          "4242-12-25 13:37", "42")

    @decorators.attr(type='negative')
    @decorators.idempotent_id('54650d60-ec17-44b7-8732-2183852789ae')
    def test_get_nonexistent_cron_trigger(self):
        self.assertRaises(exceptions.NotFound,
                          self.client.get_object,
                          'cron_triggers', 'trigger')

    @decorators.attr(type='negative')
    @decorators.idempotent_id('c663599e-5cd7-49ff-9c0f-f82a5bcc5fdb')
    def test_delete_nonexistent_trigger(self):
        self.assertRaises(exceptions.NotFound,
                          self.client.delete_obj,
                          'cron_triggers', 'trigger')

    @decorators.attr(type='negative')
    @decorators.idempotent_id('d1328d2b-5dc2-4521-93ec-d734d5fb4df7')
    def test_create_two_cron_triggers_with_same_name(self):
        tr_name = 'trigger'
        self.client.create_cron_trigger(
            tr_name, self.wf_name, None, '5 * * * *')
        self.assertRaises(exceptions.Conflict,
                          self.client.create_cron_trigger,
                          tr_name, self.wf_name, None, '5 * * * *')

    @decorators.attr(type='negative')
    @decorators.idempotent_id('3e51fc44-ce38-4653-9e4e-08b077a1dbc5')
    def test_create_two_cron_triggers_with_same_pattern(self):
        self.client.create_cron_trigger(
            'trigger1',
            self.wf_name,
            None,
            '5 * * * *',
            "4242-12-25 13:37",
            "42"
        )
        self.assertRaises(
            exceptions.Conflict,
            self.client.create_cron_trigger,
            'trigger2',
            self.wf_name,
            None,
            '5 * * * *',
            "4242-12-25 13:37",
            "42"
        )

    @decorators.attr(type='negative')
    @decorators.idempotent_id('6f3c08f3-9498-410e-a44b-4f9c6c971405')
    def test_invalid_cron_pattern_not_enough_params(self):
        self.assertRaises(exceptions.BadRequest,
                          self.client.create_cron_trigger,
                          'trigger', self.wf_name, None, '5 *')

    @decorators.attr(type='negative')
    @decorators.idempotent_id('26cb52e7-1ef3-45a2-a870-1baec2382c55')
    def test_invalid_cron_pattern_out_of_range(self):
        self.assertRaises(exceptions.BadRequest,
                          self.client.create_cron_trigger,
                          'trigger', self.wf_name, None, '88 * * * *')

        self.assertRaises(exceptions.BadRequest,
                          self.client.create_cron_trigger,
                          'trigger', self.wf_name, None, '* 88 * * *')

        self.assertRaises(exceptions.BadRequest,
                          self.client.create_cron_trigger,
                          'trigger', self.wf_name, None, '* * * 35 *')

        self.assertRaises(exceptions.BadRequest,
                          self.client.create_cron_trigger,
                          'trigger', self.wf_name, None, '* * * * 15')
